#ifndef GOAP_ACTION_HPP
#define GOAP_ACTION_HPP

#include "actor_world_state.hpp"
#include "state.hpp"
#include <godot_cpp/variant/string.hpp>
#include <godot_cpp/templates/vector.hpp>
#include <godot_cpp/templates/hash_map.hpp>

#define GOAP_ACTION(Name_) \
public: \
    _FORCE_INLINE_ static gd::String get_static_class() { return #Name_; }\
    _FORCE_INLINE_ virtual gd::String get_class() const override { return #Name_; }\
private:

namespace goap {
typedef int ActionID;

/*! A goap action construct.
 *
 * Child classes an be registered with `ActionDB::register_action`
 */
class Action {
    friend class ActionDB;
public:
    static gd::String get_static_class() { return "Action"; }
    virtual gd::String get_class() const { return "Action"; }

    virtual ~Action();
    virtual State *get_apply_state() const = 0;

    bool is_completed(ActorWorldState *context) const;
    bool is_possible(ActorWorldState *context) const;

    WorldState const &get_required() const;
    WorldState const &get_effects() const;

    ActionID get_id() const;
protected:
    Action() = default;

    virtual bool procedural_is_possible(ActorWorldState *context) const;
    virtual bool procedural_is_completed(ActorWorldState *context) const;
protected:
    WorldState blocking_required{};
    WorldState required{};
    WorldState effects{};
    bool only_proc_is_completed{false};
private:
    ActionID id{-1};
};
}

#endif // !GOAP_ACTION_HPP
