#ifndef RTS_PLAYER_HPP
#define RTS_PLAYER_HPP

#include "godot_cpp/classes/input_event.hpp"
#include "utils/player.hpp"
#include "utils/player_input.hpp"
#include <godot_cpp/classes/node3d.hpp>

namespace gd = godot;

class RTSPlayer : public gd::Node3D,
                  public utils::IPlayer {
    GDCLASS(RTSPlayer, godot::Node3D);
    static void _bind_methods();
public:
    virtual void _process(double delta_time) override;

    virtual void setup_player_input(utils::PlayerInput *input) override;
    virtual Node *to_node() override;
    virtual void spawn_at_position(gd::Transform3D const &at) override;
private:
    void move(gd::Vector2 motion);

    // input functions
    void on_mouse_horizontal(gd::Ref<gd::InputEvent> event, float value);
    void on_mouse_vertical(gd::Ref<gd::InputEvent> event, float value);
    void on_direction_horizontal(gd::Ref<gd::InputEvent> event, float value);
    void on_direction_vertical(gd::Ref<gd::InputEvent> event, float value);
    void on_rotate_horizontal(gd::Ref<gd::InputEvent> event, float value);
    void on_lclick(gd::Ref<gd::InputEvent> event, float value);
    void on_rclick(gd::Ref<gd::InputEvent> event, float value);
    void on_mclick(gd::Ref<gd::InputEvent> event, float value);

    // getters
    gd::Vector3 get_forward_direction() const;
    gd::Vector3 get_left_direction() const;
private:
    bool mmb_down{false};
    bool rmb_down{false};
    gd::Vector2 camera_mouse_motion{0.f, 0.f};
    float camera_mouse_rotation{0.f};

    gd::Vector2 camera_keys_motion{0.f, 0.f};
    float camera_keys_rotation{0.f};

    float const camera_keys_speed{10.f};
    float const camera_keys_rotation_speed{1.f};
    float const camera_mouse_speed{0.01f};
    float const camera_mouse_rotation_speed{-0.003f};
};

#endif // !RTS_PLAYER_HPP
